
// NOTICE: 

//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



#include "../include/filter.h"
#include "resource.h"
#include <math.h>
#include <stdio.h>
#include <time.h>
#include <commctrl.h>
//#include <windows.h>
#include "VdubGfxLib.h"



#define SCALE	256         /* scale factor: do floats with int math */
#define	SHIFT   16		   		 /* shift of scale 2^shift=scale */


// ---- PROTOTYPES ----------

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, 
	int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

int filter01RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int startProc(FilterActivation *fa, const FilterFunctions *ff);
int configProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
int configProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void filter_string(const FilterActivation *fa, const FilterFunctions *ff, char *buf);
long paramProc(FilterActivation *fa, const FilterFunctions *ff);



// ---- VARIABLES ----------

typedef struct MyFilterData {
//	unsigned int	tab[3][3][256];    /* multiply lookup table */
	int tab[14][256];
	int cc[256][256][256];
	int y_offset;
	int _clamp[1025];
	int mode;
	int color;
	int count;
	int outOnly;
	int yuv;
	IFilterPreview *ifp;


} MyFilterData;


struct FilterDefinition filterDef_tutorial = {
    NULL, NULL, NULL,
    "testPatterns",			// name
    "Make some ramps and compare them back in.", // desc
    "trevlac",					// maker
    NULL,
    sizeof(MyFilterData),   // inst_data_size
	NULL, NULL,
    filter01RunProc,		    // runProc
    paramProc,      
	configProc,
	filter_string,
	startProc,					// init stuff
	NULL,
	NULL,NULL
};

static FilterDefinition *fd_tutorial;


// ---- FUNCTIONS ----------


//*** VirtualdubFilterModuleInit2 ****

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm,
 const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_tutorial = ff->addFilter(fm, &filterDef_tutorial, sizeof(FilterDefinition))))
        return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}


//*** VirtualdubFilterModuleDeinit ****

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm,
 const FilterFunctions *ff) {
    ff->removeFilter(fd_tutorial);
}


//*** filter01RunProc ****

int filter01RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
    PixDim w, h;
    Pixel *src, *dst, old_pixel, hotColor, r, g, b, save_pixel;
	int rx, gx, bx, y, cb, cr, multi;
	unsigned int cc, bad, patColors;

	cc=0;
	bad=0;
	patColors = 0;
	save_pixel = 0xffffff;


	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	int *clamp = &mfd->_clamp[320];

	multi = 0;
	hotColor = 0x00FF00;

	switch(mfd->color) {
	case 0: //red
		rx=1; gx=0; bx=0;
//		hotColor = 0x00FFFF;
		patColors = 162;
		break;

	case 1: //magenta
		rx=1; gx=0; bx=1;
//		hotColor = 0x00FF00;
		patColors = 175;
		break;

	case 2: //blue
		rx=0; gx=0; bx=1;
//		hotColor = 0xFFFF00;
		patColors = 136;
		break;

	case 3: //cyan
		rx=0; gx=1; bx=1;
//		hotColor = 0xFF0000;
		patColors = 207;
		break;

	case 4: //green
		rx=0; gx=1; bx=0;
		hotColor = 0xFF00FF;
		patColors = 198;
		break;

	case 5: //yellow
		rx=1; gx=1; bx=0;
//		hotColor = 0x0000FF;
		patColors = 226;
		break;

	case 6: //grey
		rx=1; gx=1; bx=1;
//		hotColor = 0x00FF00;
		patColors = 220;
		break;

	case 7: //RMB
		//rx=1; gx=1; bx=1;
//		hotColor = 0x00FF00;
		multi = 1;
		patColors = 24385;
		break;

	case 8: //BCG
		//rx=1; gx=1; bx=1;
//		hotColor = 0xFF0000;
		multi = 2;
		patColors = 27076;
		break;

	case 9: //GYR
		//rx=1; gx=1; bx=1;
//		hotColor = 0x0000FF;
		multi = 3;
		patColors = 28999;
		break;
	}

	if(mfd->outOnly==1) mfd->mode =0;

	if(mfd->yuv) {  //--------- YUV mode

		if(mfd->mode==1 ) { //------ Compare Input YUV

			src = fa->src.data;
			dst = fa->dst.data;

			h = fa->dst.h;
			do {
					w = fa->dst.w;
				if(!multi) { //-------- Base Colors
				
					r = ((rx * (h-1)>>1)) ;
					g = ((gx * (h-1)>>1));
					b = (bx * (h-1)>>1);

					//*** Go to YCbCr
						y = (int) ( (mfd->tab[5][r] + mfd->tab[6][g] + mfd->tab[7][b] + (SCALE<<7))  >>SHIFT) + 16; 
						cb = (int) ( (mfd->tab[8][r] + mfd->tab[9][g] + mfd->tab[10][b] + (SCALE<<7))  >>SHIFT) + 128; 
						cr = (int) ( (mfd->tab[11][r] + mfd->tab[12][g] + mfd->tab[13][b] + (SCALE<<7))  >>SHIFT) + 128; 

					//*** And Back
						r = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[1][cr] + (SCALE<<7) )  >>SHIFT),0)];
						b = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[4][cb] + (SCALE<<7) )  >>SHIFT),0)];
						g = clamp[(int) max(( (mfd->tab[0][y] - mfd->tab[2][cr] - mfd->tab[3][cb] + (SCALE<<7) ) >>SHIFT),0)];

					do {
						old_pixel = *src++;
						mfd->cc[(old_pixel>>16) &255 ][(old_pixel>>8) &255][(old_pixel) &255] = 1;
						*dst++ = ((( (r<<16) + (g<<8) + b) & 0x00FFFFFF) == (old_pixel & 0x00FFFFFF))?old_pixel:hotColor;
						bad += (*(dst -1) == hotColor)?1:0;

					} while(--w);

				} else { //-------- Multi Hues

					do {

						switch(multi) {
						case 1:

							//--- RMB
							r = (((h-1)>>1) ) ;
							g = (((w-1)>>2) );
							b = ((w-1)>>1) ;
							break;

						case 2:
							//--- BCG
							r = (((w-1)>>2)) ;
							g = (((h-1)>>1));
							b = ((w-1)>>1) ;
							break;
							
						case 3:
							//--- GYR
							r = (((h-1)>>1) ) ;
							g = (((w-1)>>1) );
							b = ((w-1)>>2) ;
						}


						//*** Go to YCbCr
							y = (int) ( (mfd->tab[5][r] + mfd->tab[6][g] + mfd->tab[7][b] + (SCALE<<7))  >>SHIFT) + 16; 
							cb = (int) ( (mfd->tab[8][r] + mfd->tab[9][g] + mfd->tab[10][b] + (SCALE<<7))  >>SHIFT) + 128; 
							cr = (int) ( (mfd->tab[11][r] + mfd->tab[12][g] + mfd->tab[13][b] + (SCALE<<7))  >>SHIFT) + 128; 

						//*** And Back
							r = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[1][cr] + (SCALE<<7) )  >>SHIFT),0)];
							b = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[4][cb] + (SCALE<<7) )  >>SHIFT),0)];
							g = clamp[(int) max(( (mfd->tab[0][y] - mfd->tab[2][cr] - mfd->tab[3][cb] + (SCALE<<7) ) >>SHIFT),0)];

						old_pixel = *src++;						
						mfd->cc[(old_pixel>>16) &255 ][(old_pixel>>8) &255][(old_pixel) &255] = 1;
						*dst++ = ((( (r<<16) + (g<<8) + b) & 0x00FFFFFF) == (old_pixel & 0x00FFFFFF))?old_pixel:hotColor;
						bad += (*(dst -1) == hotColor)?1:0;
							
					} while(--w);
				} //--- End Multi if 
				

					src = (Pixel *)((char *)src + fa->src.modulo);
					dst = (Pixel *)((char *)dst + fa->dst.modulo);
				} while(--h);

		} else {	//---------- Just create Output YUV

			dst = fa->dst.data;

			h = fa->dst.h;
			do {
				w = fa->dst.w;

				if(!multi) { //-------- Base Colors
				
					r = ((rx * (h-1)>>1)) ;
					g = ((gx * (h-1)>>1));
					b = (bx * (h-1)>>1);

					//*** Go to YCbCr
						y = (int) ( (mfd->tab[5][r] + mfd->tab[6][g] + mfd->tab[7][b] + (SCALE<<7))  >>SHIFT) + 16; 
						cb = (int) ( (mfd->tab[8][r] + mfd->tab[9][g] + mfd->tab[10][b] + (SCALE<<7))  >>SHIFT) + 128; 
						cr = (int) ( (mfd->tab[11][r] + mfd->tab[12][g] + mfd->tab[13][b] + (SCALE<<7))  >>SHIFT) + 128; 

					//*** And Back
						r = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[1][cr] + (SCALE<<7) )  >>SHIFT),0)];
						b = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[4][cb] + (SCALE<<7) )  >>SHIFT),0)];
						g = clamp[(int) max(( (mfd->tab[0][y] - mfd->tab[2][cr] - mfd->tab[3][cb] + (SCALE<<7) ) >>SHIFT),0)];

					do {*dst++ = (r<<16) + (g<<8) + b;} while(--w);

				} else { //-------- Multi Hues

					do {

						switch(multi) {
						case 1:

							//--- RMB
							r = (((h-1)>>1) ) ;
							g = (((w-1)>>2) );
							b = ((w-1)>>1) ;
							break;

						case 2:
							//--- BCG
							r = (((w-1)>>2)) ;
							g = (((h-1)>>1));
							b = ((w-1)>>1) ;
							break;
							
						case 3:
							//--- GYR
							r = (((h-1)>>1) ) ;
							g = (((w-1)>>1) );
							b = ((w-1)>>2) ;
						}


						//*** Go to YCbCr
							y = (int) ( (mfd->tab[5][r] + mfd->tab[6][g] + mfd->tab[7][b] + (SCALE<<7))  >>SHIFT) + 16; 
							cb = (int) ( (mfd->tab[8][r] + mfd->tab[9][g] + mfd->tab[10][b] + (SCALE<<7))  >>SHIFT) + 128; 
							cr = (int) ( (mfd->tab[11][r] + mfd->tab[12][g] + mfd->tab[13][b] + (SCALE<<7))  >>SHIFT) + 128; 

						//*** And Back
							r = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[1][cr] + (SCALE<<7) )  >>SHIFT),0)];
							b = clamp[(int) max(( (mfd->tab[0][y] + mfd->tab[4][cb] + (SCALE<<7) )  >>SHIFT),0)];
							g = clamp[(int) max(( (mfd->tab[0][y] - mfd->tab[2][cr] - mfd->tab[3][cb] + (SCALE<<7) ) >>SHIFT),0)];
						
						
						*dst++ = (r<<16) + (g<<8) + b;
							
					} while(--w);
				} //--- End Multi if 



					dst = (Pixel *)((char *)dst + fa->dst.modulo);
				} while(--h);
		}
	}else { //---------- RGB mode

		if(mfd->mode==1 ) { //------ Compare Input RGB

			src = fa->src.data;
			dst = fa->dst.data;

			h = fa->dst.h;
			do {

				w = fa->dst.w;
				if(!multi) { //-------- Base Colors
				
					r = ((rx * (h-1)>>1) << 16);
					g = ((gx * (h-1)>>1)<< 8);
					b = (bx * (h-1)>>1) & 0xff;

					patColors = 256;

					do {
						old_pixel = *src++;
						mfd->cc[(old_pixel>>16) &255 ][(old_pixel>>8) &255][(old_pixel) &255] = 1;
						*dst++ = (((r + g + b) & 0x00FFFFFF) == (old_pixel & 0x00FFFFFF))?old_pixel:hotColor;
						bad += (*(dst -1) == hotColor)?1:0;

					} while(--w);

				} else { //-------- Multi Hues

					do {

						patColors = 65536;

						switch(multi) {
						case 1:

							//--- RMB
							r = (((h-1)>>1) <<16) ;
							g = (((w-1)>>2) << 8);
							b = ((w-1)>>1) & 0xff;
							break;

						case 2:
							//--- BCG
							r = (((w-1)>>2) <<16) ;
							g = (((h-1)>>1) << 8);
							b = ((w-1)>>1) & 0xff;
							break;
							
						case 3:
							//--- GYR
							r = (((h-1)>>1) <<16) ;
							g = (((w-1)>>1) << 8);
							b = ((w-1)>>2) & 0xff;
						}
						old_pixel = *src++;
						mfd->cc[(old_pixel>>16) &255 ][(old_pixel>>8) &255][(old_pixel) &255] = 1;
						*dst++ = (((r + g + b) & 0x00FFFFFF) == (old_pixel & 0x00FFFFFF))?old_pixel:hotColor;
						bad += (*(dst -1) == hotColor)?1:0;
							
					} while(--w);
				} //--- End multi if


					src = (Pixel *)((char *)src + fa->src.modulo);
					dst = (Pixel *)((char *)dst + fa->dst.modulo);
				} while(--h);

		} else {	//------------- Just Create Output RGB

			dst = fa->dst.data;

			h = fa->dst.h;
			do {
				w = fa->dst.w;

				if(!multi) { //-------- Base Colors
				
					r = ((rx * (h-1)>>1) << 16);
					g = ((gx * (h-1)>>1)<< 8);
					b = (bx * (h-1)>>1) & 0xff;

					do {*dst++ = (r) + (g) + b;} while(--w);

				} else { //-------- Multi Hues

					do {

						switch(multi) {
						case 1:

							//--- RMB
							r = (((h-1)>>1) <<16) ;
							g = (((w-1)>>2) << 8);
							b = ((w-1)>>1) & 0xff;
							break;

						case 2:
							//--- BCG
							r = (((w-1)>>2) <<16) ;
							g = (((h-1)>>1) << 8);
							b = ((w-1)>>1) & 0xff;
							break;
							
						case 3:
							//--- GYR
							r = (((h-1)>>1) <<16) ;
							g = (((w-1)>>1) << 8);
							b = ((w-1)>>2) & 0xff;
						}
						*dst++ = (r) + (g) + b;
							
					} while(--w);
				}

					dst = (Pixel *)((char *)dst + fa->dst.modulo);
			} while(--h);
		}
	}

	if(mfd->count && mfd->mode ) {
		for(int r1=0;r1<256;r1++)
			for(int g1=0;g1<256;g1++)
				for(int b1=0;b1<256;b1++)
					cc += mfd->cc[r1][g1][b1];



		char  msg[50];

		fa->dst.RectFill(5,5,300,50,0);

		msg[sprintf( msg,"Pattern Color Count: %d  ", patColors)] = 0;
   		VGL_DrawString(fa, 10, 10, msg, 0xFFFFB700);

		msg[sprintf( msg,"Input Color Count: %d  ", cc)] = 0;
   		VGL_DrawString(fa, 10, 20, msg, 0xFFFFB700);

		msg[sprintf( msg,"Pixels Not Matching: %d  ", bad)] = 0;
   		VGL_DrawString(fa, 10, 30, msg, 0xFFFFB700);

		msg[sprintf( msg,"%% Pixels Not Matching: %d %%  ", 100*bad/(512*512))] = 0;
   		VGL_DrawString(fa, 10, 40, msg, 0xFFFFB700);

		

	}


    return 0;
}





int startProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	/*
	unsigned int	f;
	int	pv;
	//double y_matrix[3] = { 0.299, 0.587, 0.114,};	 //** 601 0-255
	double y_matrix[3] = { 0.2126, 0.7152, 0.0722,}; //** 709 0-255

	//mfd->y_offset = 16;
	//y_matrix[0] = 0.2567890625;
	//y_matrix[1] = 0.50412890625;
	//y_matrix[2] = 0.0979625;


	//--- Luma Lookup --
	for (pv = 0; pv <= 255; pv++) {
		f = SCALE * (pv );
		mfd->tab[0][0][pv] = (unsigned int)(f * (y_matrix[0]) + 0.5);
		mfd->tab[0][1][pv] = (unsigned int)(f * (y_matrix[1]) + 0.5);
		mfd->tab[0][2][pv] = (unsigned int)(f * (y_matrix[2]) + 0.5);
	}

*/
  int rng1 =255;
  int rng2 = 0;

  if(true) { rng1 = 219; rng2 = 16; }

  //if(interpolate==5) { vi.width = vi.width / 2; }

  const int crv = int(350.901*SCALE*255/rng1 + (SCALE>1));
  const int cgv = int(178.738*SCALE*255/rng1 + (SCALE>1));
  const int cgu = int(86.132*SCALE*255/rng1 + (SCALE>1));
  const int cbu = int(443.506*SCALE*255/rng1 + (SCALE>1));
  const int y = int(256*SCALE*255/rng1 + (SCALE>1));

  const int yr = int(76.544*SCALE*rng1/255 + (SCALE>1));
  const int yg = int(150.272*SCALE*rng1/255 + (SCALE>1));
  const int yb = int(29.184*SCALE*rng1/255 + (SCALE>1));

  const int ur = int(-44.182*SCALE*rng1/255 + (SCALE>1));
  const int ug = int(-86.704*SCALE*rng1/255 + (SCALE>1));
  const int ub = int(130.922*SCALE*rng1/255 + (SCALE>1));

  const int vr = int(130.922*SCALE*rng1/255 + (SCALE>1));
  const int vg = int(-109.631*SCALE*rng1/255 + (SCALE>1));
  const int vb = int(-21.291*SCALE*rng1/255 + (SCALE>1));

  //*** Build lookups

  int pv;
  for (pv = 0; pv <= 255; pv++) {

	mfd->tab[0][pv] = (int)((pv-rng2)*y); //scaled_y
	mfd->tab[1][pv] = (int)((pv-128) * crv ); //CrR
	mfd->tab[2][pv] = (int)((pv-128) * cgv ); //CrG
	mfd->tab[3][pv] = (int)((pv-128) * cgu ); //CbG
	mfd->tab[4][pv] = (int)((pv-128) * cbu ); //CbB

	mfd->tab[5][pv] = (int)((pv) * yr ); //YR
	mfd->tab[6][pv] = (int)((pv) * yg ); //YG
	mfd->tab[7][pv] = (int)((pv) * yb ); //YB

	mfd->tab[8][pv] = (int)((pv) * ur ); //
	mfd->tab[9][pv] = (int)((pv) * ug ); //
	mfd->tab[10][pv] = (int)((pv) * ub ); //

	mfd->tab[11][pv] = (int)((pv) * vr ); //
	mfd->tab[12][pv] = (int)((pv) * vg ); //
	mfd->tab[13][pv] = (int)((pv) * vb ); //

  }

  for(int r=0;r<256;r++)
	  for(int g=0;g<256;g++)
		  for(int b=0;b<256;b++)
			  mfd->cc[r][g][b]=0;

  for (pv = 0; pv <=1024; pv++) {
	  mfd->_clamp[pv] = min(max(pv-320,0),255);
  }

    return 0;
}



BOOL CALLBACK  configDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);
//	HWND hWnd;

    switch(msg) {
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;

			if(mfd->outOnly==1) mfd->mode==0;
			EnableWindow(GetDlgItem(hdlg, IDC_IN),(mfd->outOnly==0));

            CheckDlgButton(hdlg, IDC_OUT, (mfd->mode==0 )?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_IN, (mfd->mode==1 )?BST_CHECKED:BST_UNCHECKED);

            CheckDlgButton(hdlg, IDC_R, mfd->color==0?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_M, mfd->color==1?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_B, mfd->color==2?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_C, mfd->color==3?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_G, mfd->color==4?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_Y, mfd->color==5?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_GREY, mfd->color==6?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_RMB, mfd->color==7?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_BCG, mfd->color==8?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_GYR, mfd->color==9?BST_CHECKED:BST_UNCHECKED);

            CheckDlgButton(hdlg, IDC_COUNT, mfd->count==1?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_YUV, mfd->yuv==1?BST_CHECKED:BST_UNCHECKED);



			if (mfd->ifp)
				mfd->ifp->InitButton(GetDlgItem(hdlg, IDC_PREVIEW));


            return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
                if(IsDlgButtonChecked(hdlg, IDC_OUT)) mfd->mode=0;
                if(IsDlgButtonChecked(hdlg, IDC_IN)) mfd->mode=1;
                if(IsDlgButtonChecked(hdlg, IDC_R)) mfd->color=0;
                if(IsDlgButtonChecked(hdlg, IDC_M)) mfd->color=1;
                if(IsDlgButtonChecked(hdlg, IDC_B)) mfd->color=2;
                if(IsDlgButtonChecked(hdlg, IDC_C)) mfd->color=3;
                if(IsDlgButtonChecked(hdlg, IDC_G)) mfd->color=4;
                if(IsDlgButtonChecked(hdlg, IDC_Y)) mfd->color=5;
                if(IsDlgButtonChecked(hdlg, IDC_GREY)) mfd->color=6;
                if(IsDlgButtonChecked(hdlg, IDC_RMB)) mfd->color=7;
                if(IsDlgButtonChecked(hdlg, IDC_BCG)) mfd->color=8;
                if(IsDlgButtonChecked(hdlg, IDC_GYR)) mfd->color=9;

                mfd->count = !!(IsDlgButtonChecked(hdlg,IDC_COUNT));
                mfd->yuv = !!(IsDlgButtonChecked(hdlg,IDC_YUV));

                EndDialog(hdlg, 0);
                return TRUE;
            case IDCANCEL:
                if(IsDlgButtonChecked(hdlg, IDC_OUT)) mfd->mode=0;
                if(IsDlgButtonChecked(hdlg, IDC_IN)) mfd->mode=1;
                EndDialog(hdlg, 1);
                return FALSE;

			case IDC_PREVIEW:
				if (mfd->ifp) {
					mfd->ifp->Toggle(hdlg);
					//mfd->ifp->RedoFrame();
				}

				return TRUE;

			case IDC_OUT:
			case IDC_IN:
			case IDC_R:
			case IDC_M:
			case IDC_B:
			case IDC_C:
			case IDC_G:
			case IDC_Y:
			case IDC_GREY:
			case IDC_RMB:
			case IDC_BCG:
			case IDC_GYR:
			case IDC_COUNT:
			case IDC_YUV:
                if(IsDlgButtonChecked(hdlg, IDC_OUT) ) mfd->mode=0;
                if(IsDlgButtonChecked(hdlg, IDC_IN) ) mfd->mode=1;
                if(IsDlgButtonChecked(hdlg, IDC_R)) mfd->color=0;
                if(IsDlgButtonChecked(hdlg, IDC_M)) mfd->color=1;
                if(IsDlgButtonChecked(hdlg, IDC_B)) mfd->color=2;
                if(IsDlgButtonChecked(hdlg, IDC_C)) mfd->color=3;
                if(IsDlgButtonChecked(hdlg, IDC_G)) mfd->color=4;
                if(IsDlgButtonChecked(hdlg, IDC_Y)) mfd->color=5;
                if(IsDlgButtonChecked(hdlg, IDC_GREY)) mfd->color=6;
                if(IsDlgButtonChecked(hdlg, IDC_RMB)) mfd->color=7;
                if(IsDlgButtonChecked(hdlg, IDC_BCG)) mfd->color=8;
                if(IsDlgButtonChecked(hdlg, IDC_GYR)) mfd->color=9;

                mfd->count = !!(IsDlgButtonChecked(hdlg,IDC_COUNT));
                mfd->yuv = !!(IsDlgButtonChecked(hdlg,IDC_YUV));

				if (mfd->ifp) mfd->ifp->RedoFrame();

				return TRUE;


            }
            break;
    }

    return FALSE;
}

int configProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {

	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	mfd->ifp = fa->ifp;


	mfd->outOnly = 0;
	if(fa->src.w != fa->dst.w || fa->src.h != fa->dst.h)
		mfd->outOnly = 1;

    return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER), hwnd,
            (DLGPROC) configDlgProc, (LPARAM)fa->filter_data);
}


void filter_string(const FilterActivation *fa, const FilterFunctions *ff, char *buf) {

	MyFilterData *mfd = (MyFilterData *)fa->filter_data;


}


long paramProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    fa->dst.w = 512;
	fa->dst.h = 512;

	fa->dst.pitch = (fa->dst.w*4 + 7) & -8;

    return FILTERPARAM_SWAP_BUFFERS;
}

